import { customEvent } from '@genesislcap/foundation-events';
import { Form } from '@genesislcap/foundation-forms';
import { html, ref, when } from '@microsoft/fast-element';
import { List } from '../list';
import { ModalFormType, CrudMenuPosition, CrudAction } from '../types';
List;
Form;
/**
 * Default add button for Entity Management screen
 */
const addButtonTemplate = (prefix) => html `
  ${when((x) => x.createEvent, html `
      <${prefix}-button class="crud-button" appearance="neutral" @click=${(x) => x.createEntity()}>
        <${prefix}-icon size="lg" name="plus"></${prefix}-icon>
        ${(x) => x.getTitleBasedOnActionOrType(ModalFormType.Create)}
      </${prefix}-button>
    `)}
`;
/**
 * Default crud buttons for Entity Management screen
 */
export const crudButtonsTemplate = (prefix, position) => html `
  ${when((x) => x.crudMenuPosition === position || x.hasContentInSlot(`crud-${position}`), html `
      <section
        part=${() => `crud-buttons-wrapper crud-buttons-wrapper-${position}`}
        class=${() => `crud-buttons-wrapper crud-buttons-wrapper-${position}`}
      >
        <slot name=${() => `crud-${position}-before`}></slot>
        <slot name=${() => `crud-${position}`}>
          <nav
            ${ref('crudMenuWrapper')}
            part=${() => `crud-buttons crud-buttons-${position}`}
            class=${() => `crud-buttons crud-buttons-${position}`}
          >
            ${when((x) => !x.crudMenuStyle.includes('actions-'), html `
                ${addButtonTemplate(prefix)}
                ${when((x) => x.crudMenuPosition !== CrudMenuPosition.Column, html `
                    ${when((x) => x.updateEvent && !x.hideEdit, html `
                      <${prefix}-button class="crud-button" appearance="neutral" title="Select an item from the grid to enable editing" @click=${(x, y) => x.emitCrud('edit')} disabled=${(x) => (x.hasSelectedEntity ? null : 'disabled')}>
                        <${prefix}-icon size="lg" name="pen"></${prefix}-icon>
                          ${(x) => x.getTitleBasedOnActionOrType(ModalFormType.Update)}
                      </${prefix}-button>
                      `)}
                    ${when((x) => x.deleteEvent && !x.hideDelete, html `
                        <${prefix}-button class="crud-button" appearance="neutral" title="Select an item from the grid to enable deleting" @click=${(x, y) => x.emitCrud('delete')} disabled=${(x) => (x.hasSelectedEntity ? null : 'disabled')}>
                          <${prefix}-icon size="lg" name="trash-alt"></${prefix}-icon>
                          ${(x) => x.getTitleBasedOnActionOrType(CrudAction.Delete)}
                        </${prefix}-button>
                      `)}
                  `)}
              `)}
          </nav>
        </slot>
        <slot name=${`crud-${position}-after`}></slot>
      </section>
    `)}
`;
/**
 * Default header crud buttons area for Entity Management screen
 * @internal
 */
export const headerCrudAreaTemplate = (prefix) => html `
  ${crudButtonsTemplate(prefix, CrudMenuPosition.Top)}
  ${when((x) => x.crudMenuPosition === CrudMenuPosition.Column &&
    !x.hasContentInSlot('crud-top') &&
    !x.hasContentInSlot('crud-bottom'), html `
      <section
        part=${() => `crud-buttons-wrapper crud-buttons-wrapper-column`}
        class=${() => `crud-buttons-wrapper crud-buttons-wrapper-column`}
      >
        ${addButtonTemplate(prefix)}
      </section
    `)}
`;
/**
 * Default header for Entity Management screen
 * @internal
 */
export const defaultHeader = (prefix) => html `
  ${when((x) => x.title, html `
      <h3 class="header-title">${(x) => x.title}</h3>
    `)}
  ${headerCrudAreaTemplate(prefix)}
`;
/**
 * Header with search-bar for Entity Management screen
 * @internal
 */
export const searchBarHeader = (prefix) => html `
  ${when((x) => x.title, html `
      <h3 class="header-title">${(x) => x.title}</h3>
    `)}
  <div class="search-container">
    <${prefix}-search-bar
      :options="${(x) => x.searchBarConfig}"
      @selectionChange=${(x, ctx) => x.searchChanged(customEvent(ctx))}
    ></${prefix}-search-bar>
    ${headerCrudAreaTemplate(prefix)}
  </div>
`;
/**
 * Defines the html of the entity component as a ViewTemplate object
 * @typeParam EntityManagement - the {@link EntityManagement | class} the template is defined for
 */
export const getPrefixedEntities = (prefix) => html `
  <div class="container">
    <slot name="header" part="header">${(x) => x.headerTempalate}</slot>
    <entity-list
      title=${(x) => x.title}
      async-add=${(x) => x.asyncAdd}
      async-remove=${(x) => x.asyncRemove}
      async-update=${(x) => x.asyncUpdate}
      enable-cell-flashing=${(x) => x.enableCellFlashing}
      enable-row-flashing=${(x) => x.enableRowFlashing}
      persist-column-state-key=${(x) => x.persistColumnStateKey}
      row-selection=${(x) => x.rowSelection}
      size-columns-to-fit=${(x) => x.sizeColumnsToFit}
      enable-filter-bar=${(x) => x.enableFilterBar}
      resourceName=${(x) => x.resourceName}
      :deleteEvent=${(x) => x.deleteEvent}
      :updateEvent=${(x) => x.updateEvent}
      :gridOptions=${(x) => x.gridOptions}
      :columns=${(x) => x.columns}
      :datasourceConfig=${(x) => x.datasourceConfig}
      design-system-prefix=${prefix}
      data-test-id="entity-list"
      hide-edit=${(x) => x.shouldHideEditInColumn}
      hide-delete=${(x) => x.shouldHideDeleteInColumn}
      header-case-type=${(x) => x.headerCaseType}
      datasource-type=${(x) => x.datasourceType}
      @selectionChanged=${(x, c) => x.selectionChanged(c.event)}
    ></entity-list>
    <slot name="footer" slot="footer">${crudButtonsTemplate(prefix, CrudMenuPosition.Bottom)}</slot>
    <${prefix}-modal
      ${ref('editEntityModal')}
      class="edit-modal"
      position=${(x) => x.modalPosition}
      :onCloseCallback=${(x) => x.closeModal.bind(x)}
      data-test-id="edit-entity-modal"
    >
      <div slot="top" data-test-id="dialog-title">${(x) => x.modalFormTitle}</div>

      <slot name="edit">
        ${when((x) => x.editModalVisible, html `
            <foundation-form
              part="form"
              :renderers=${(x) => x.formRenderers}
              resourceName=${(x) => x.formResourceName}
              :data=${(x) => x.editedEntity}
              design-system-prefix=${prefix}
              :uischema=${(x) => x.formUiSchema}
              readonly=${(x) => x.readonly}
              @submit-success=${(x, c) => x.submitEntityChanges()}
            ></foundation-form>
          `)}
      </slot>
    </${prefix}-modal>
  </div>
`;
/** @internal */
export const template = html `
  ${(x) => getPrefixedEntities(x.prefix)}
`;
